from typing import TYPE_CHECKING

import pytest

from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import DataToken, FetchTokenRequest, FetchTokenResponse
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.users.conftest import FakeFetchTokenResponse
    from tests.api.utils import E2EConfig

# pyright: reportPrivateUsage=false


def test_fetch_session_token_calls_post_and_parses_response(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_fetch_token_response: "FakeFetchTokenResponse",
):
    api = UsersApi(session=mocker.Mock())
    api._post = mocker.Mock(return_value=mocker.Mock(json=fake_fetch_token_response.build().model_dump))

    send_data = {"mobi": faker.email(), "code": str(faker.random_number(digits=6))}
    result = api.fetch_session_token(payload=FetchTokenRequest.model_validate(send_data))

    assert isinstance(result, FetchTokenResponse)
    api._post.assert_called_once_with(endpoint="/api/1/users/signupByPhoneOrEmail", json=send_data)


@pytest.mark.parametrize("url", list(BooxUrl))
def test_users_api_fetch_session_token_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_fetch_token_response: "FakeFetchTokenResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    mock_response = mocker.Mock()
    mock_response.json.return_value = fake_fetch_token_response.build().model_dump()
    mock_response.raise_for_status.return_value = mock_response
    mock_client.post.return_value = mock_response

    with Boox(client=mock_client, base_url=url) as boox:
        send_data = {"mobi": faker.email(), "code": str(faker.random_number(digits=6))}
        payload = FetchTokenRequest.model_validate(send_data)
        result = boox.users.fetch_session_token(payload=payload)

    mock_client.post.assert_called_once_with(url.value + "/api/1/users/signupByPhoneOrEmail", json=send_data)
    mock_response.json.assert_called_once()
    assert isinstance(result, FetchTokenResponse)
    assert isinstance(result.data, DataToken)


@e2e
@pytest.mark.order(1)
def test_fetch_session_token_e2e(config: "E2EConfig"):
    if not config.verification_code:
        pytest.skip("Verification code was either not obtained or not set")
    payload = FetchTokenRequest.model_validate({"mobi": config.email_address, "code": config.verification_code})

    with Boox(base_url=config.domain) as boox:
        result = boox.users.fetch_session_token(payload=payload)

    assert result.data.token
    config.token = result.data.token



from typing import TYPE_CHECKING

import pytest

from boox.api.core import TokenMissingError
from boox.core import Boox
from boox.models.config_users import SyncTokenResponse
from boox.models.enums import BooxUrl
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.config_users.conftest import FakeSyncTokenResponse
    from tests.api.utils import E2EConfig


def test_sync_token_raises_token_missing_error(mock_boox: Boox):
    with pytest.raises(TokenMissingError, match="Bearer token is required to call this method"):
        mock_boox.config_users.synchronize_token()


@pytest.mark.parametrize("url", list(BooxUrl))
def test_config_users_api_sync_token_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_sync_token_response: "FakeSyncTokenResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    token = faker.uuid4()
    return_value = fake_sync_token_response.build()
    mock_response = mocker.Mock()
    mock_response.json = mocker.Mock(return_value=return_value.model_dump())
    mock_response.raise_for_status.return_value = mock_response
    mock_client.get.return_value = mock_response

    with Boox(client=mock_client, base_url=url, token=token) as boox:
        result = boox.config_users.synchronize_token()

    expected_url = url.value + "/api/1/configUsers/one"
    mock_client.get.assert_called_once_with(expected_url)
    mock_response.json.assert_called_once()
    assert isinstance(result, SyncTokenResponse)
    assert not result.data


@e2e
def test_synchronize_token_e2e(config: "E2EConfig"):
    if not config.token:
        pytest.skip("Token was either not obtained or not set")

    with Boox(base_url=config.domain, token=config.token) as boox:
        result = boox.config_users.synchronize_token()

    assert not result.data
    assert result.token_expired_at



from polyfactory.factories.pydantic_factory import ModelFactory
from polyfactory.pytest_plugin import register_fixture

from boox.models.config_users import SyncTokenResponse


@register_fixture
class FakeSyncTokenResponse(ModelFactory[SyncTokenResponse]):
    __check_model__ = True



from collections.abc import Iterator
from contextlib import suppress
from typing import TYPE_CHECKING

import pytest
from polyfactory.factories.pydantic_factory import ModelFactory
from polyfactory.pytest_plugin import register_fixture

from boox.models.users import (
    DeviceInfoResponse,
    FetchTokenResponse,
    SendVerifyResponse,
    SyncSessionTokenResponse,
    UserInfoResponse,
)
from tests.api.utils import EmailProvider

if TYPE_CHECKING:
    from tests.api.utils import E2EConfig


@pytest.fixture(scope="session")
def email(config: "E2EConfig") -> Iterator[EmailProvider]:
    """An email provider for connecting to an SMTP server.

    Useful for getting the verification code.
    At the end of the session all messages in the inbox are cleaned-up.

    Yields:
        EmailProvider: a testing-only wrapper on httpx.Client.
    """
    provider = EmailProvider(config)
    yield provider
    with suppress(ValueError):
        provider.cleanup_inbox()


@register_fixture
class FakeSendVerifyResponse(ModelFactory[SendVerifyResponse]):
    __check_model__ = True  # TODO: Remove when `polyfactory` lib is updated to v3


@register_fixture
class FakeFetchTokenResponse(ModelFactory[FetchTokenResponse]):
    __check_model__ = True


@register_fixture
class FakeSyncSessionTokenResponse(ModelFactory[SyncSessionTokenResponse]):
    __check_model__ = True


@register_fixture
class FakeUserInfoResponse(ModelFactory[UserInfoResponse]):
    __check_model__ = True


@register_fixture
class FakeDeviceInfoResponse(ModelFactory[DeviceInfoResponse]):
    __check_model__ = True



import datetime
from typing import TYPE_CHECKING

from boox.models.config_users import SyncTokenResponse

if TYPE_CHECKING:
    from faker import Faker


def test_sync_token_response_parses_token_expiration_date_correctly(faker: "Faker"):
    token_expiry = faker.date_time(tzinfo=datetime.UTC).replace(microsecond=0)
    data = SyncTokenResponse.model_validate({
        "data": None,
        "message": faker.pystr(),
        "result_code": faker.random_digit(),
        "tokenExpiredAt": int(token_expiry.timestamp()),
    })
    assert not data.data
    assert data.token_expired_at == token_expiry



import json
import re
from typing import TYPE_CHECKING

import pytest
from pydantic import ValidationError

from boox.models.users import (
    DataToken,
    FetchTokenRequest,
    SendVerifyCodeRequest,
    soft_validate_email,
)

if TYPE_CHECKING:
    from faker import Faker


def test_soft_validate_email_returns_true_for_valid_email(faker: "Faker"):
    is_email = soft_validate_email(faker.email())
    assert is_email


def test_soft_validate_email_returns_false_for_invalid_email(faker: "Faker"):
    is_email = soft_validate_email(faker.email(domain=faker.random_lowercase_letter()))
    assert not is_email


def test_validation_fails_when_mobi_is_empty_string():
    with pytest.raises(ValidationError, match="String should have at least 6 characters"):
        SendVerifyCodeRequest.model_validate({"mobi": ""})


def test_validation_requires_area_code_for_phone_number(faker: "Faker"):
    with pytest.raises(ValidationError, match="area_code must be provided if phone method is used"):
        SendVerifyCodeRequest.model_validate({"mobi": str(faker.msisdn())})


def test_validation_fails_when_area_code_does_not_match_pattern(faker: "Faker"):
    with pytest.raises(ValidationError, match="String should match pattern"):
        SendVerifyCodeRequest.model_validate({
            "mobi": str(faker.msisdn()),
            "area_code": faker.country_calling_code().replace("+", "00"),
        })


def test_validation_fails_when_mobi_is_neither_email_nor_phone(faker: "Faker"):
    with pytest.raises(ValidationError, match="mobi field must either be an e-mail or a phone number"):
        SendVerifyCodeRequest.model_validate({"mobi": faker.email(domain=faker.random_lowercase_letter())})


def test_validation_fails_when_email_and_area_code_are_both_provided(faker: "Faker"):
    with pytest.raises(ValidationError, match="mobi and area_code are mutually exclusive"):
        SendVerifyCodeRequest.model_validate({"mobi": faker.email(), "area_code": faker.country_calling_code()})


def test_validation_allows_email_without_area_code(faker: "Faker"):
    assert SendVerifyCodeRequest.model_validate({"mobi": faker.email()})


def test_validation_fails_when_verification_code_does_not_match_pattern(faker: "Faker"):
    with pytest.raises(ValidationError, match="String should match pattern"):
        FetchTokenRequest.model_validate({"mobi": faker.email(), "code": str(faker.random_number(digits=7))})


def test_validation_succeeds_when_verification_code_matches_pattern(faker: "Faker"):
    assert FetchTokenRequest.model_validate({"mobi": faker.email(), "code": str(faker.random_number(digits=6))})


def test_token_str_is_masked(faker: "Faker"):
    data = DataToken.model_validate({"token": faker.uuid4()})
    assert re.compile(r"\*+").fullmatch(str(data.token))


def test_token_is_public_in_model_dump(faker: "Faker"):
    value = faker.uuid4()
    data = DataToken.model_validate({"token": value})
    dumped = data.model_dump()
    assert dumped.get("token") == value


def test_token_is_public_in_json_dump(faker: "Faker"):
    data = {"token": faker.uuid4()}
    model = DataToken.model_validate(data)
    dumped = model.model_dump_json()
    assert json.dumps(data, separators=(", ", ":")) == dumped



"""Example on how to get device(s) data."""

from typing import TYPE_CHECKING, cast

from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import DataDevice

if TYPE_CHECKING:
    from boox.models.users import DeviceInfoResponse

TOKEN = cast(str, ...)  # Just to make this example easier


with Boox(base_url=BooxUrl.PUSH, token=TOKEN) as boox:
    response: "DeviceInfoResponse" = boox.users.get_device_info()

# For `data` field, this response returns an array of objects.
# These objects are wrapped by `DataDevice` model
_data: tuple["DataDevice", ...] = response.data

for _item in _data:
    print(_item.height)



import datetime
import re
from contextlib import suppress
from typing import Annotated, Any, ClassVar, Literal, Self

from pydantic import (
    BaseModel,
    ConfigDict,
    EmailStr,
    Field,
    PlainSerializer,
    SecretStr,
    StringConstraints,
    model_validator,
    validate_email,
)
from pydantic_core import PydanticCustomError

from boox.models.base import BaseResponse, BaseSyncResponse

# NOTE: If having `Any`, it means that it lacks information on type based on my internal testing.
# If you have some additional data, please contribute.


def soft_validate_email(value: str) -> bool:
    with suppress(PydanticCustomError):
        return bool(validate_email(value))
    return False


class BaseVerificationModel(BaseModel):
    """The base request body for verification and authentication requests.

    Attributes:
        area_code (str | None): Optional. Required if `mobi` is an e-mail.
        mobi (str): Required. Either mobile number or e-mail.

    """

    model_config: ClassVar[ConfigDict] = ConfigDict(frozen=True, str_strip_whitespace=True)

    mobi: Annotated[str, StringConstraints(min_length=6)]
    area_code: Annotated[str, StringConstraints(min_length=2, pattern=r"^\+\d+$")] | None = Field(default=None)

    @model_validator(mode="after")
    def check_area_code(self) -> Self:
        is_phone = bool(re.fullmatch(r"\d+", self.mobi))
        is_email = bool(soft_validate_email(self.mobi))

        if not is_phone and not is_email:
            raise ValueError("mobi field must either be an e-mail or a phone number")
        if is_phone and not self.area_code:
            raise ValueError("area_code must be provided if phone method is used")
        if is_email and self.area_code:
            raise ValueError("mobi and area_code are mutually exclusive")
        return self


class SendVerifyCodeRequest(BaseVerificationModel):
    """A request body for POST users/sendVerifyCode.

    Please refer to BaseVerificationModel for more attributes.

    Attributes:
        scene (str): Unknown.
        verify (str): Unknown.

    There are basically 2 important usages:
    - send code to mobile,
    - send code to email.

    Examples:
        >>> SendVerifyCodeRequest(area_code="+48", mobi="600123456")  # mobile
        >>> SendVerifyCodeRequest(mobi="foo@bar.com")  # email
    """

    model_config: ClassVar[ConfigDict] = ConfigDict(frozen=True, str_strip_whitespace=True)

    scene: str = ""
    verify: str = ""


class SendVerifyResponse(BaseResponse[str]):
    """Base response type with data being a str."""


class FetchTokenRequest(BaseVerificationModel):
    """A request body for POST users/signupByPhoneOrEmail.

    Attributes:
        code (str): The verification code received via either mobile or e-mail.

    Note:
        Verification code is 6 digits long, but is passed as a string.
    """

    code: Annotated[str, StringConstraints(pattern=r"^\d{6}$")]


class DataToken(BaseModel):
    token: Annotated[SecretStr, PlainSerializer(lambda v: v.get_secret_value(), return_type=str, when_used="always")]


class FetchTokenResponse(BaseResponse[DataToken]):
    """A response, with token under data key."""


class DataSession(BaseModel):
    channels: tuple[Any, ...]
    cookie_name: str
    expires: datetime.datetime
    session_id: SecretStr


class SyncSessionTokenResponse(BaseSyncResponse[DataSession]):
    """A response, with information about expiry dates, with an emphasis on the session."""


class DataUser(BaseModel):
    model_config: ClassVar[ConfigDict] = ConfigDict(serialize_by_alias=True)

    access_token: Any | None = Field(alias="accessToken")
    area_code: Annotated[str, StringConstraints(min_length=2, pattern=r"^\+\d+$")]
    avatar: Any | None
    avatar_url: Any | None = Field(alias="avatarUrl")
    children: tuple[Any, ...]
    consecutive_usage_days: int = Field(alias="consecutiveUsageDays")
    device_limit: int
    email: EmailStr
    gift_count: int = Field(alias="giftCount")
    google_id: Any | None
    huawei_id: Any | None
    id: int
    login_type: Literal["phone", "email"]
    nickname: str | None
    oauth_id: Any | None
    otp_auth_url: Any | None
    otp_enabled: bool
    otp_verified: bool
    parent: Any | None
    phone: str | None
    role_value: int = Field(alias="roleValue")
    sex: Any | None
    storage_limit: int
    storage_used: int
    total_doc_count: int = Field(alias="totalDocCount")
    total_usage_days: int = Field(alias="totalUsageDays")
    uid: str
    vip_cloud: int
    vip_cloud_end: Any | None
    vip_cloud_start: Any | None
    wechat_id: Any | None


class UserInfoResponse(BaseSyncResponse[DataUser]):
    """A response with basic account information."""


class DataDevice(BaseModel):
    model_config: ClassVar[ConfigDict] = ConfigDict(serialize_by_alias=True)

    installation_map: dict[str, Any] = Field(alias="installationMap")
    id: int
    model: str
    device_unique_id: str = Field(alias="deviceUniqueId")
    mac_address: str = Field(alias="macAddress")
    number: Any | None
    version_int: int = Field(alias="versionInt")
    version_release: str = Field(alias="versionRelease")
    width: int
    height: int
    brand: str
    system: Any | None
    channel: Any | None
    build_id: str = Field(alias="buildId")
    modfingerprintel: Any | None
    timezone: Any | None
    name: Any | None
    details: str
    current_account_id: Any | None = Field(alias="currentAccountId")
    lock: int
    im_account: str = Field(alias="imAccount")
    created_at: datetime.datetime
    updated_at: datetime.datetime
    org_id: Any | None
    latest_login_time: datetime.datetime = Field(alias="latestLoginTime")
    latest_logout_time: datetime.datetime = Field(alias="latestLogoutTime")
    login_status: str = Field(alias="loginStatus")


class DeviceInfoResponse(BaseSyncResponse[tuple[DataDevice, ...]]):
    """A response with basic device information.

    Since this response can contain information about many devices,
    the `data` field is wrapped in a tuple of `DataDevice`(s).
    """



"""Example on how to synchronize session token.

While this method is not required at all, especially if you write short scripts,
it makes sense to use it occasionally to make sure that you don't use an expired token.
But you might want to use it only if you rely on the session_id.
"""

import datetime
from typing import TYPE_CHECKING, cast

from boox.core import Boox
from boox.models.enums import BooxUrl

if TYPE_CHECKING:
    from pydantic import SecretStr

    from boox.models.users import SyncSessionTokenResponse

TOKEN = cast(str, ...)  # Just to make this example easier


with Boox(base_url=BooxUrl.EUR, token=TOKEN) as boox:
    response: "SyncSessionTokenResponse" = boox.users.synchronize_session_token()

_session_id: "SecretStr" = response.data.session_id
_expires: datetime.datetime = response.data.expires



"""Example on how to get user data.

Quite a useful method if you'd like to see whether the changes to the account were applied.
"""

from typing import TYPE_CHECKING, cast

from boox.core import Boox
from boox.models.enums import BooxUrl

if TYPE_CHECKING:
    from boox.models.users import UserInfoResponse

TOKEN = cast(str, ...)  # Just to make this example easier


with Boox(base_url=BooxUrl.PUSH, token=TOKEN) as boox:
    response: "UserInfoResponse" = boox.users.get_user_info()


# This response contains a lot of data about your account
# Most notable are listed below.
# login_type - how you logged in
_login_type: str = response.data.login_type

# nickname - your custom name for the account
_nickname: str | None = response.data.nickname

# storage_used and storage_limit
_storage_used: int = response.data.storage_used
_storage_limit: int = response.data.storage_limit



from typing import TYPE_CHECKING

import pytest

from boox.api.core import TokenMissingError
from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import DataUser, UserInfoResponse
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.users.conftest import FakeUserInfoResponse
    from tests.api.utils import E2EConfig

# pyright: reportPrivateUsage=false


def test_get_user_info_calls_get_and_parses_response(
    mocker: "MockerFixture",
    fake_user_info_response: "FakeUserInfoResponse",
):
    api = UsersApi(session=mocker.Mock())
    api._get = mocker.Mock(return_value=mocker.Mock(json=fake_user_info_response.build().model_dump))

    result = api.get_user_info()

    api._get.assert_called_once_with(endpoint="/api/1/users/me")
    assert isinstance(result, UserInfoResponse)
    assert isinstance(result.data, DataUser)


@pytest.mark.parametrize("url", list(BooxUrl))
def test_users_api_get_user_info_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_user_info_response: "FakeUserInfoResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    mock_response = mocker.Mock()
    mock_response.json.return_value = fake_user_info_response.build().model_dump()
    mock_response.raise_for_status.return_value = mock_response
    mock_client.get.return_value = mock_response

    with Boox(client=mock_client, base_url=url, token=faker.uuid4()) as boox:
        result = boox.users.get_user_info()

    mock_client.get.assert_called_once_with(url.value + "/api/1/users/me")
    mock_response.json.assert_called_once()
    mock_response.raise_for_status.assert_called_once()
    assert isinstance(result, UserInfoResponse)
    assert isinstance(result.data, DataUser)


def test_get_user_info_raises_token_missing_error(mock_boox: Boox):
    with pytest.raises(TokenMissingError, match="Bearer token is required to call this method"):
        mock_boox.users.get_user_info()


@e2e
def test_get_user_info_e2e(config: "E2EConfig"):
    if not config.token:
        pytest.skip("Token was either not obtained or not set")

    with Boox(base_url=config.domain, token=config.token) as boox:
        result = boox.users.get_user_info()

    assert result.data.area_code
    assert result.data.login_type == "email"
    assert result.data.email == config.email_address



from typing import TYPE_CHECKING

import pytest
from pydantic import ValidationError

from boox.models.base import BaseResponse, BaseSyncResponse

if TYPE_CHECKING:
    from faker import Faker


def test_base_response_str_format(faker: "Faker"):
    message = faker.pystr()
    result_code = faker.random_digit()
    response = BaseResponse(data=None, message=message, result_code=result_code)
    assert str(response) == f"<{result_code}: {message}>"


def test_subclass_response_repr_format(faker: "Faker"):
    message = faker.pystr()
    result_code = faker.random_digit()

    class DummyResponse(BaseResponse[None]): ...

    response = DummyResponse(data=None, message=message, result_code=result_code)
    assert repr(response) == f"DummyResponse(<{result_code}: {message}>)"


def test_base_sync_subclass_raises_error_if_no_token_expiry_date_provided(faker: "Faker"):
    class DummySyncResponse(BaseSyncResponse[None]): ...

    with pytest.raises(ValidationError, match="Field required"):
        DummySyncResponse.model_validate({"data": None, "message": faker.pystr(), "result_code": faker.random_digit()})



from collections.abc import Iterator
from typing import TYPE_CHECKING

import pytest

from boox.core import Boox
from boox.models.enums import BooxUrl
from tests.api.utils import E2EConfig

if TYPE_CHECKING:
    from unittest.mock import Mock


@pytest.fixture(scope="session")
def config() -> E2EConfig:
    return E2EConfig()


@pytest.fixture
def mock_boox(mock_client: "Mock") -> Iterator[Boox]:
    with Boox(client=mock_client, base_url=BooxUrl.EUR) as boox:
        yield boox



import re
from typing import TYPE_CHECKING

import pytest

from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import SendVerifyCodeRequest, SendVerifyResponse
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.users.conftest import FakeSendVerifyResponse
    from tests.api.utils import E2EConfig, EmailProvider

# pyright: reportPrivateUsage=false


def test_send_verification_code_calls_post_and_parses_response(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_send_verify_response: "FakeSendVerifyResponse",
):
    api = UsersApi(session=mocker.Mock())
    api._post = mocker.Mock(return_value=mocker.Mock(json=fake_send_verify_response.build().model_dump))

    send_data = {"mobi": faker.email()}
    result = api.send_verification_code(payload=SendVerifyCodeRequest.model_validate(send_data))

    assert isinstance(result, SendVerifyResponse)
    api._post.assert_called_once_with(endpoint="/api/1/users/sendVerifyCode", json=send_data)


@pytest.mark.parametrize("url", list(BooxUrl))
def test_users_api_send_verification_code_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_send_verify_response: "FakeSendVerifyResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    mock_response = mocker.Mock()
    mock_response.json.return_value = fake_send_verify_response.build().model_dump()
    mock_response.raise_for_status.return_value = mock_response
    mock_client.post.return_value = mock_response

    with Boox(client=mock_client, base_url=url) as boox:
        send_data = {"mobi": faker.email()}
        payload = SendVerifyCodeRequest.model_validate(send_data)
        result = boox.users.send_verification_code(payload=payload)

    mock_client.post.assert_called_once_with(url.value + "/api/1/users/sendVerifyCode", json=send_data)
    mock_response.json.assert_called_once()
    mock_response.raise_for_status.assert_called_once()
    assert isinstance(result, SendVerifyResponse)


@e2e
@pytest.mark.order(0)
def test_send_verification_code_e2e(config: "E2EConfig", email: "EmailProvider"):
    payload = SendVerifyCodeRequest.model_validate({"mobi": config.email_address})

    with Boox(base_url=config.domain) as boox:
        result = boox.users.send_verification_code(payload=payload)

    assert result.data == "ok"
    message = email.get_newest_message()
    match = re.compile(r"^The code is (?P<code>\d{6}) for account verification from BOOX\.").match(message)
    assert match, "Did not match the received email"
    config.verification_code = match.group("code")



from typing import TYPE_CHECKING

import pytest

from boox.api.core import TokenMissingError
from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import DataDevice, DeviceInfoResponse
from tests.api.utils import E2EConfig
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.users.conftest import FakeDeviceInfoResponse

# pyright: reportPrivateUsage=false


def test_get_device_info_parses_response_correctly(
    mocker: "MockerFixture",
    fake_device_info_response: "FakeDeviceInfoResponse",
):
    api = UsersApi(session=mocker.Mock())
    api._get = mocker.Mock(return_value=mocker.Mock(json=fake_device_info_response.build().model_dump))

    result = api.get_device_info()

    api._get.assert_called_once_with(endpoint="/api/1/users/getDevice")
    assert isinstance(result, DeviceInfoResponse)
    assert isinstance(result.data, tuple)
    assert all(isinstance(d, DataDevice) for d in result.data)


def test_get_user_info_raises_token_missing_error(mock_boox: Boox):
    with pytest.raises(TokenMissingError, match="Bearer token is required to call this method"):
        mock_boox.users.get_device_info()


@pytest.mark.parametrize("url", list(BooxUrl))
def test_users_api_get_device_info_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_device_info_response: "FakeDeviceInfoResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    mock_response = mocker.Mock()
    mock_response.json.return_value = fake_device_info_response.build().model_dump()
    mock_response.raise_for_status.return_value = mock_response
    mock_client.get.return_value = mock_response

    with Boox(client=mock_client, base_url=url, token=faker.uuid4()) as boox:
        result = boox.users.get_device_info()

    mock_client.get.assert_called_once_with(url.value + "/api/1/users/getDevice")
    mock_response.json.assert_called_once()
    mock_response.raise_for_status.assert_called_once()
    assert isinstance(result, DeviceInfoResponse)
    assert isinstance(result.data, tuple)
    assert all(isinstance(d, DataDevice) for d in result.data)


@e2e
def test_get_device_info_e2e(config: "E2EConfig"):
    if not config.token:
        pytest.skip("Token was either not obtained or not set")

    with Boox(base_url=config.domain, token=config.token) as boox:
        result = boox.users.get_device_info()

    assert isinstance(result.data, tuple)
    assert all(d.id for d in result.data)



from typing import TYPE_CHECKING

import pytest

from boox.api.core import TokenMissingError
from boox.api.users import UsersApi
from boox.core import Boox
from boox.models.enums import BooxUrl
from boox.models.users import DataSession, SyncSessionTokenResponse
from tests.api.utils import E2EConfig
from tests.conftest import e2e

if TYPE_CHECKING:
    from unittest.mock import Mock

    from faker import Faker
    from pytest_mock import MockerFixture

    from tests.api.users.conftest import FakeSyncSessionTokenResponse

# pyright: reportPrivateUsage=false


def test_sync_session_token_calls_get_and_parses_response(
    mocker: "MockerFixture",
    fake_sync_session_token_response: "FakeSyncSessionTokenResponse",
):
    api = UsersApi(session=mocker.Mock())
    api._get = mocker.Mock(return_value=mocker.Mock(json=fake_sync_session_token_response.build().model_dump))

    result = api.synchronize_session_token()

    api._get.assert_called_once_with(endpoint="/api/1/users/syncToken")
    assert isinstance(result, SyncSessionTokenResponse)
    assert isinstance(result.data, DataSession)


def test_sync_session_token_raises_token_missing_error(mock_boox: Boox):
    with pytest.raises(TokenMissingError, match="Bearer token is required to call this method"):
        mock_boox.users.synchronize_session_token()


@pytest.mark.parametrize("url", list(BooxUrl))
def test_users_api_sync_session_token_parses_response_correctly(
    mocker: "MockerFixture",
    faker: "Faker",
    fake_sync_session_token_response: "FakeSyncSessionTokenResponse",
    mock_client: "Mock",
    url: BooxUrl,
):
    mock_response = mocker.Mock()
    mock_response.json.return_value = fake_sync_session_token_response.build().model_dump()
    mock_response.raise_for_status.return_value = mock_response
    mock_client.get.return_value = mock_response

    with Boox(client=mock_client, base_url=url, token=faker.uuid4()) as boox:
        result = boox.users.synchronize_session_token()

    mock_client.get.assert_called_once_with(url.value + "/api/1/users/syncToken")
    mock_response.json.assert_called_once()
    mock_response.raise_for_status.assert_called_once()
    assert isinstance(result, SyncSessionTokenResponse)
    assert isinstance(result.data, DataSession)


@e2e
def test_synchronize_session_token_e2e(config: E2EConfig):
    if not config.token:
        pytest.skip("Token was either not obtained or not set")

    with Boox(base_url=config.domain, token=config.token) as boox:
        result = boox.users.synchronize_session_token()

    assert result.data.expires



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from boox.core import Boox

# pyright: reportPrivateUsage=false


def test_config_users_api_has_access_to_boox(mock_boox: "Boox"):
    assert mock_boox.config_users._session is mock_boox
